#import "JEN_MenuAppMethods.h"
#import <Carbon/Carbon.h>

@implementation JEN_MenuAppMethods

// Updates the modification date of the app itself to get the system to recognize the change to the background-only status.
+ (void)touchBundleDate {
	[[NSFileManager defaultManager] changeFileAttributes:[NSDictionary dictionaryWithObject:[NSDate date] forKey:NSFileModificationDate] atPath:[[NSBundle mainBundle] bundlePath]];
}

// This method will load an AppleScript and call a specific handler from that script with the specified string parameter.
// Other parameters and parameter types can be added as necessary.
+ (BOOL)runASHandlerInScript:(NSString *)scriptName handler:(NSString *)handlerName parameters:(NSString *)parameters {
	int pid = [[NSProcessInfo processInfo] processIdentifier];
	NSString *compiledScriptPath = [[NSBundle mainBundle] pathForResource:scriptName ofType:@"scpt" inDirectory:@"Scripts"];
	NSDictionary *error = nil;
	NSAppleScript *script = [[NSAppleScript alloc] initWithContentsOfURL:[NSURL fileURLWithPath:compiledScriptPath] error:&error];
	if (error) {
		if (script) [script release];
		NSLog(@"(1) error: %@", error);
		return NO;
	}
	NSAppleEventDescriptor *arguments = [[NSAppleEventDescriptor alloc] initListDescriptor];
	[arguments insertDescriptor:[NSAppleEventDescriptor descriptorWithString:parameters] atIndex:0];
	NSAppleEventDescriptor *targetAddress = [[NSAppleEventDescriptor alloc] initWithDescriptorType:typeKernelProcessID bytes:&pid length:sizeof(pid)];
	NSAppleEventDescriptor *event = [[NSAppleEventDescriptor alloc] initWithEventClass:kASAppleScriptSuite eventID:kASSubroutineEvent targetDescriptor:targetAddress returnID:kAutoGenerateReturnID transactionID:kAnyTransactionID];
	[event setParamDescriptor:[NSAppleEventDescriptor descriptorWithString:[handlerName lowercaseString]] forKeyword:keyASSubroutineName];
	[event setParamDescriptor:arguments forKeyword:keyDirectObject];
	[script executeAppleEvent:event error:&error];
	[script release];
	[arguments release];
	[targetAddress release];
	[event release];
	if (error) {
		NSLog(@"(2) error: %@", error);
		return NO;
	}
	return YES;
}

@end
