property debug_me : false

property script_author : "Jonathan Nathan, JNSoftware LLC"
property script_name : "MenuApp"
property script_version : "1.3"
property script_release_date_string : "October 30, 2009"
property script_release_date : date script_release_date_string
property script_copyright : "Use subject to terms of the GPL"
property script_url : "http://www.jonn8.com"

property script_email : "mailto:JNSoftware%20LLC%20%3Cinfo%40jonn8.com%3E?subject=MenuApp%20(v" & script_version & ")"
property script_contribution_url : "https://order.kagi.com/?ELJ"

property first_run : true
property run_plead : true
property background_only : true

property status_item_controller : missing value
property menu_item_controller : missing value
property status_menu : missing value
property dynamic_submenu : missing value
property my_path : missing value

property menu_title : "MenuApp"
property icon_path : ""
property title_display : 1

on will finish launching the_object
	set object_name to name of the_object as string
	my update_status("will finish launching " & object_name)
	if first_run then set icon_path to (path for resource "updateImage_1.png")
	my register_settings()
	my read_settings()
	
	-- The path to the application is required if you want to change the app from a background only app to a regular app.
	set my_path to (path to me as string)
	
	-- These next two calls will instantiate the custom Objective-C classes written for this project.
	set status_item_controller to (call method "init" of (call method "alloc" of class "JEN_StatusItemController"))
	set menu_item_controller to (call method "init" of (call method "alloc" of class "JEN_MenuItemController"))
	
	-- Let's assign the menus that will be used:
	set status_menu to main menu's menu 1 whose name of it = "status_menu"
	set dynamic_submenu to status_menu's menu item 3's sub menu
	
	-- Give the dynamic menu some intial menu items with the action "my_action_1".
	-- This action is hard-coded in the JEN_MenuItemController class and the menu_item_controller AppleScript.
	-- You could modify these two files and add any action you wish to be called for the menu.
	-- The caveat here is that every menu item in the submenu will have the same action
	-- in your AppleScript, this requires you to differentiate between the items based on their title,
	-- not their name, and act accordingly.
	set current_date to (current date) as Unicode text
	set the_titles to {"Created at Launch 1 (" & current_date & ")", "Created at Launch 2 (" & current_date & ")", "Created at Launch 3 (" & current_date & ")"}
	my reload_menu(dynamic_submenu, the_titles, "my_action_1")
end will finish launching

on launched the_object
	my update_status("launched " & (name of the_object as string))
	activate
	show window "main"
	if first_run then display window "license" attached to window "main"
	
	-- If the app is background only, let's add the status menu by calling the proper method with the status_menu.
	if background_only then call method "createStatusItemWithMenu:" of status_item_controller with parameter status_menu
end launched

on should quit after last window closed the_object
	return (not background_only)
end should quit after last window closed

-- Set up the initial interface.
on awake from nib the_object
	set object_name to name of the_object as string
	my update_status("awake from nib " & object_name)
	if object_name = "main" then
		tell the_object
			if first_run then center
			set state of button "background_only" to (background_only as integer)
			set contents of text field "menu_title" to menu_title
			if (icon_path  "") then set image of image view "icon" to (load image icon_path)
			set current row of matrix "title_display" to title_display
		end tell
		
	else if object_name = "about" then
		tell the_object
			tell box 1
				set contents of text field "title" to (script_name & " v" & script_version)
				set contents of text field "author" to ("by " & script_author)
				set contents of text field "release" to ("Released " & ((month of script_release_date) as string) & " " & ((day of script_release_date) as string) & ", " & ((year of script_release_date) as string))
				set contents of text field "copyright" to (script_copyright)
				set state of button "run_plead" to ((offset of (("" & run_plead)'s character 1) in "ft") - 1)
			end tell
		end tell
		
	end if
end awake from nib

on choose menu item the_object
	activate
	set object_name to name of the_object as string
	my update_status("choose menu item " & object_name)
	
	if object_name = "about" then
		show window "main"
		display window "about" attached to window "main"
		
	else if object_name = "show_main" then
		show window "main"
		
	else if object_name = "reload" then
		my reload_menu_now()
		
	end if
end choose menu item

on clicked the_object
	set object_name to name of the_object as string
	my update_status("clicked " & object_name)
	
	if object_name = "decline" then
		close panel window "license"
		quit
		
	else if object_name = "accept" then
		set first_run to false
		my write_settings()
		close panel window "license"
		
	else if object_name = "dialog_cancel" then
		set the_win to name of window of the_object
		close panel window the_win
		
	else if object_name = "dialog_ok" then
		set the_win to name of window of the_object
		close panel window the_win
		
	else if object_name = "about_webpage" then
		open location script_url
		
	else if object_name = "about_email" then
		open location script_email
		
	else if object_name = "contribute" then
		open location script_contribution_url
		
	else if object_name = "run_plead" then
		set run_plead to true
		if state of the_object = 0 then set run_plead to false
		my write_settings()
		
	else if object_name = "reload" then
		my reload_menu_now()
		
	else if object_name = "background_only" then
		set background_only to ((state of the_object) = 1)
		my toggle_LSUIElement()
		
	else if object_name = "title_display" then
		set title_display to current row of the_object
		my write_settings()
		call method "updateDisplay" of status_item_controller
		
	else if object_name = "animation_display" then
		set animation_display to current row of the_object
		call method "updateAnimation:" of status_item_controller with parameter animation_display
		
	else if object_name = "choose_icon" then
		set image_path to POSIX path of (choose file with prompt "Choose an Image for the Icon:")
		set image of (image view "icon" of window "main") to (load image image_path)
		set icon_path to image_path
		my write_settings()
		call method "updateDisplay" of status_item_controller
		
	end if
end clicked

on will quit the_object
	my quit_prep()
	return true
end will quit

on will open the_object
	try
		set {x, y} to (get size of window "main")
		set size of the_object to {x, y - 22}
	end try
	return true
end will open

on should close the_object
	-- this works around a bug that releases a window when it closes so instead of closing the window, we just hide it
	tell the_object to hide
	return false
end should close

-- This is called if you drag and drop an image into the image view in the main window.
on conclude drop the_object drag info drag_info
	try
		if ((types of pasteboard of drag_info) contains "file names") then
			set preferred type of pasteboard of drag_info to "file names"
			set dropped_image_path to item 1 of (get contents of pasteboard of drag_info)
			set image of the_object to (load image dropped_image_path)
			set icon_path to dropped_image_path
			my write_settings()
			call method "updateDisplay" of status_item_controller
		end if
	on error e
		log e
	end try
end conclude drop

-- This will update the menu title when you are done editing the title in the main window.
-- Use tab or return to commit the change.
on end editing the_object
	set menu_title to string value of the_object
	my write_settings()
	call method "updateDisplay" of status_item_controller
end end editing

-- Make sure the about panel (and others) close so we can actually quit.
on close_all_panels()
	try
		set the_panels to windows whose class = panel and visible = true
		repeat with this_panel in the_panels
			try
				if name of this_panel  "main" then close panel this_panel
			on error
				close panel this_panel
			end try
		end repeat
	end try
end close_all_panels

-- We're quitting so let's clean everything up.
on quit_prep()
	my close_all_panels()
	-- If the status menu item was called, let's try to release it.
	try
		call method "releaseStatusItem" of status_item_controller
	end try
	my write_settings()
	if run_plead then my plead()
	set quitting to true
end quit_prep

on display_error(the_title, the_error)
	activate
	beep
	my update_status("Error: " & the_error)
	display alert the_title as critical message the_error default button "OK" attached to window "main"
end display_error

-- The next several handlers are used to deal with the app's preferences.
on get_setting_names()
	return {"first_run", "run_plead", "background_only", "menu_title", "icon_path", "title_display"}
end get_setting_names

on get_setting_values()
	return {first_run, run_plead, background_only, menu_title, icon_path, title_display}
end get_setting_values

on set_setting_values(v)
	set {first_run, run_plead, background_only, menu_title, icon_path, title_display} to v
end set_setting_values

on register_settings()
	my update_status("registering settings...")
	set {n, v} to {my get_setting_names(), my get_setting_values()}
	tell user defaults
		repeat with i from 1 to (count n)
			make new default entry at end of default entries with properties {name:(item i of n), contents:(item i of v)}
		end repeat
		register
	end tell
	my update_status("settings registered")
end register_settings

on read_settings()
	my update_status("reading settings...")
	set {n, v} to {my get_setting_names(), {}}
	tell user defaults
		repeat with i from 1 to (count n)
			set end of v to contents of default entry (item i of n)
		end repeat
	end tell
	my set_setting_values(v)
	my update_status("settings read")
end read_settings

on write_settings()
	my update_status("writing settings...")
	set {n, v} to {my get_setting_names(), my get_setting_values()}
	tell user defaults
		repeat with i from 1 to (count n)
			set contents of default entry (item i of n) to (item i of v)
		end repeat
	end tell
	my update_status("settings written")
end write_settings

on update_status(the_message)
	if debug_me = true then log the_message
end update_status

on plead()
	tell application "System Events"
		activate
		set the_button to button returned of (display dialog "If you found this useful, and we hope you did, please consider contributing to our web hosting and general development costs. (You may disable this reminder in the About box.)" buttons {"Maybe Later", "Contribute Now"} default button 2 with icon 1)
	end tell
	if the_button = "Contribute Now" then open location script_contribution_url
end plead

on reload_menu_now()
	tell window "main"
		try
			set menu_item_count to integer value of text field "menu_item_count"
		on error
			set menu_item_count to 3
		end try
		set the_action to title of button "reload_action"
	end tell
	set current_date to (current date) as string
	set the_titles to {}
	repeat with i from 1 to menu_item_count
		set end of the_titles to "Reloaded Menu Item " & i & " (" & current_date & ")"
	end repeat
	my reload_menu(dynamic_submenu, the_titles, the_action)
end reload_menu_now

-- This will call the appropriate method in the JEN_MenuItemController class to add menu items to 
-- the target menu with the appropriate titles and action.
on reload_menu(the_menu, the_titles, the_action)
	try
		delete the_menu's menu items
	end try
	call method "populateMenu:withData:withAction:" of menu_item_controller with parameters {the_menu, the_titles, the_action}
end reload_menu

-- This routine will cause the app to attempt to modify itself to change the internal LSUIElement bit.
on toggle_LSUIElement()
	if background_only then
		set {LSUIElement, the_message_add} to {"1", "run in the background with only the status menu visible"}
	else
		set {LSUIElement, the_message_add} to {"0", "appear in the Dock & have full application menus"}
	end if
	try
		if my_path does not end with ":" then set my_path to my_path & ":"
		do shell script "defaults write " & (quoted form of POSIX path of (my_path & "Contents:Info")) & " LSUIElement " & LSUIElement
		-- Touch the bundle so that the Launch Services database will respect the change:
		call method "touchBundleDate" of class "JEN_MenuAppMethods"
		set the_error to ("This application will now " & the_message_add & ". Quit & relaunch to take advantage of the new setting." & return & return & "Please Note: Your system should automatically recognize this change. If, however, the change is not recognized the next time you launch this application, you must force your system to recognize the change. To do so, quit this application, then, in the Finder, move the application icon to the desktop (or some folder other than its current location), and launch again." & return & return & "Mac OS X uses a cache for application settings that doesnt always get updated properly when settings are modified.")
		my display_error("Application Modified", the_error)
	on error the_error
		my display_error("Application Modified Error", the_error)
	end try
end toggle_LSUIElement
